@@ -27,13 +27,15 @@ class Project(resource.Resource, tag.TagMixin):
     allow_list = True
     commit_method = 'PATCH'
 
+    _allow_unknown_attrs_in_body = True
+
     _query_mapping = resource.QueryParameters(
         'domain_id',
         'is_domain',
         'name',
         'parent_id',
         is_enabled='enabled',
-        **tag.TagMixin._tag_query_parameters
+        **tag.TagMixin._tag_query_parameters,
     )
 
     # Properties
@@ -62,54 +64,72 @@ class Project(resource.Resource, tag.TagMixin):
 
     def assign_role_to_user(self, session, user, role):
         """Assign role to user on project"""
-        url = utils.urljoin(self.base_path, self.id, 'users',
-                            user.id, 'roles', role.id)
-        resp = session.put(url,)
+        url = utils.urljoin(
+            self.base_path, self.id, 'users', user.id, 'roles', role.id
+        )
+        resp = session.put(
+            url,
+        )
         if resp.status_code == 204:
             return True
         return False
 
     def validate_user_has_role(self, session, user, role):
         """Validates that a user has a role on a project"""
-        url = utils.urljoin(self.base_path, self.id, 'users',
-                            user.id, 'roles', role.id)
-        resp = session.head(url,)
+        url = utils.urljoin(
+            self.base_path, self.id, 'users', user.id, 'roles', role.id
+        )
+        resp = session.head(
+            url,
+        )
         if resp.status_code == 204:
             return True
         return False
 
     def unassign_role_from_user(self, session, user, role):
         """Unassigns a role from a user on a project"""
-        url = utils.urljoin(self.base_path, self.id, 'users',
-                            user.id, 'roles', role.id)
-        resp = session.delete(url,)
+        url = utils.urljoin(
+            self.base_path, self.id, 'users', user.id, 'roles', role.id
+        )
+        resp = session.delete(
+            url,
+        )
         if resp.status_code == 204:
             return True
         return False
 
     def assign_role_to_group(self, session, group, role):
         """Assign role to group on project"""
-        url = utils.urljoin(self.base_path, self.id, 'groups',
-                            group.id, 'roles', role.id)
-        resp = session.put(url,)
+        url = utils.urljoin(
+            self.base_path, self.id, 'groups', group.id, 'roles', role.id
+        )
+        resp = session.put(
+            url,
+        )
         if resp.status_code == 204:
             return True
         return False
 
     def validate_group_has_role(self, session, group, role):
         """Validates that a group has a role on a project"""
-        url = utils.urljoin(self.base_path, self.id, 'groups',
-                            group.id, 'roles', role.id)
-        resp = session.head(url,)
+        url = utils.urljoin(
+            self.base_path, self.id, 'groups', group.id, 'roles', role.id
+        )
+        resp = session.head(
+            url,
+        )
         if resp.status_code == 204:
             return True
         return False
 
     def unassign_role_from_group(self, session, group, role):
         """Unassigns a role from a group on a project"""
-        url = utils.urljoin(self.base_path, self.id, 'groups',
-                            group.id, 'roles', role.id)
-        resp = session.delete(url,)
+        url = utils.urljoin(
+            self.base_path, self.id, 'groups', group.id, 'roles', role.id
+        )
+        resp = session.delete(
+            url,
+        )
         if resp.status_code == 204:
             return True
         return False
