@@ -12,6 +12,7 @@
 
 import os
 import time
+import typing as ty
 import warnings
 
 from openstack import exceptions
@@ -19,6 +20,8 @@ from openstack.image.v2 import cache as _cache
 from openstack.image.v2 import image as _image
 from openstack.image.v2 import member as _member
 from openstack.image.v2 import metadef_namespace as _metadef_namespace
+from openstack.image.v2 import metadef_object as _metadef_object
+from openstack.image.v2 import metadef_property as _metadef_property
 from openstack.image.v2 import metadef_resource_type as _metadef_resource_type
 from openstack.image.v2 import metadef_schema as _metadef_schema
 from openstack.image.v2 import schema as _schema
@@ -37,13 +40,13 @@ _RAW_PROPERTIES = ('is_protected', 'protected', 'tags')
 
 def _get_name_and_filename(name, image_format):
     # See if name points to an existing file
-    if os.path.exists(name):
+    if os.path.exists(name) and os.path.isfile(name):
         # Neat. Easy enough
         return os.path.splitext(os.path.basename(name))[0], name
 
     # Try appending the disk format
     name_with_ext = '.'.join((name, image_format))
-    if os.path.exists(name_with_ext):
+    if os.path.exists(name_with_ext) and os.path.isfile(name):
         return os.path.basename(name), name_with_ext
 
     return name, None
@@ -87,7 +90,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.image.v2.image.Image`
             instance.
         :param bool ignore_missing: When set to ``False``,
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the metadef namespace does not exist.
         :returns: ``None``
         """
@@ -98,7 +101,7 @@ class Proxy(proxy.Proxy):
         cache = self._get_resource(_cache.Cache, None)
         return cache.queue(self, image_id)
 
-    def clear_cache(self, target):
+    def clear_cache(self, target='both'):
         """Clear all images from cache, queue or both
 
         :param target: Specify which target you want to clear
@@ -330,8 +333,8 @@ class Proxy(proxy.Proxy):
                 validate_checksum=validate_checksum,
                 use_import=use_import,
                 stores=stores,
-                all_stores=stores,
-                all_stores_must_succeed=stores,
+                all_stores=all_stores,
+                all_stores_must_succeed=all_stores_must_succeed,
                 **image_kwargs,
             )
         else:
@@ -340,8 +343,6 @@ class Proxy(proxy.Proxy):
             image_kwargs['name'] = name
             image = self._create(_image.Image, **image_kwargs)
 
-        self._connection._get_cache(None).invalidate()
-
         return image
 
     def import_image(
@@ -505,7 +506,7 @@ class Proxy(proxy.Proxy):
         """
         warnings.warn(
             "upload_image is deprecated. Use create_image instead.",
-            os_warnings.OpenStackDeprecationWarning,
+            os_warnings.RemovedInSDK50Warning,
         )
         # container_format and disk_format are required to be set
         # on the image by the time upload_image is called, but they're not
@@ -597,12 +598,10 @@ class Proxy(proxy.Proxy):
             self.log.debug("Image creation failed", exc_info=True)
             raise
         except Exception as e:
-            raise exceptions.SDKException(
-                "Image creation failed: {message}".format(message=str(e))
-            )
+            raise exceptions.SDKException(f"Image creation failed: {str(e)}")
 
     def _make_v2_image_params(self, meta, properties):
-        ret = {}
+        ret: ty.Dict = {}
         for k, v in iter(properties.items()):
             if k in _INT_PROPERTIES:
                 ret[k] = int(v)
@@ -733,7 +732,6 @@ class Proxy(proxy.Proxy):
         }
 
         glance_task = self.create_task(**task_args)
-        self._connection.list_images.invalidate(self)
         if wait:
             start = time.time()
 
@@ -770,7 +768,6 @@ class Proxy(proxy.Proxy):
                 # Clean up after ourselves. The object we created is not
                 # needed after the import is done.
                 self._connection.delete_object(container, name)
-                self._connection.list_images.invalidate(self)
             return image
         else:
             return glance_task
@@ -835,7 +832,7 @@ class Proxy(proxy.Proxy):
             image is associated with. If specified, the image will only be
             deleted from the specified store.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the image does not exist.
             When set to ``True``, no exception will be set when
             attempting to delete a nonexistent image.
@@ -853,7 +850,7 @@ class Proxy(proxy.Proxy):
 
         :param name_or_id: The name or ID of a image.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the resource does not exist.
             When set to ``True``, None will be returned when
             attempting to find a nonexistent resource.
@@ -872,7 +869,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.image.v2.image.Image` instance.
 
         :returns: One :class:`~openstack.image.v2.image.Image`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(_image.Image, image)
@@ -950,7 +947,7 @@ class Proxy(proxy.Proxy):
         for k, v in iter(kwargs.items()):
             if v and k in ['ramdisk', 'kernel']:
                 v = self._connection.get_image_id(v)
-                k = '{0}_id'.format(k)
+                k = f'{k}_id'
             properties[k] = v
 
         img_props = image.properties.copy()
@@ -963,8 +960,6 @@ class Proxy(proxy.Proxy):
 
         self.update_image(image, **img_props)
 
-        self._connection.list_images.invalidate(self._connection)
-
         return True
 
     def add_tag(self, image, tag):
@@ -1023,7 +1018,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.image.v2.image.Image` instance that the member
             is part of. This is required if ``member`` is an ID.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the member does not exist. When set to ``True``, no exception will
             be set when attempting to delete a nonexistent member.
 
@@ -1046,7 +1041,7 @@ class Proxy(proxy.Proxy):
             the value can be the ID of a image or a
             :class:`~openstack.image.v2.image.Image` instance.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the resource does not exist.
             When set to ``True``, None will be returned when
             attempting to find a nonexistent resource.
@@ -1069,7 +1064,7 @@ class Proxy(proxy.Proxy):
             The value can be the ID of a image or a
             :class:`~openstack.image.v2.image.Image` instance.
         :returns: One :class:`~openstack.image.v2.member.Member`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         member_id = resource.Resource._get_id(member)
@@ -1141,7 +1136,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
             instance.
         :param bool ignore_missing: When set to ``False``,
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the metadef namespace does not exist.
         :returns: ``None``
         """
@@ -1164,7 +1159,7 @@ class Proxy(proxy.Proxy):
 
         :returns: One
             :class:`~~openstack.image.v2.metadef_namespace.MetadefNamespace`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound` when no
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
             resource can be found.
         """
         return self._get(
@@ -1177,7 +1172,7 @@ class Proxy(proxy.Proxy):
 
         :returns: A generator object of metadef namespaces
         :rtype: :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._list(_metadef_namespace.MetadefNamespace, **query)
@@ -1205,6 +1200,134 @@ class Proxy(proxy.Proxy):
             **attrs,
         )
 
+    # ====== METADEF OBJECT ======
+    def create_metadef_object(self, namespace, **attrs):
+        """Create a new object from namespace
+
+        :param namespace: The value can be either the name of a metadef
+            namespace or a
+            :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            instance.
+        :param dict attrs: Keyword arguments which will be used to create
+            a :class:`~openstack.image.v2.metadef_object.MetadefObject`,
+            comprised of the properties on the Metadef object class.
+
+        :returns: A metadef namespace
+        :rtype: :class:`~openstack.image.v2.metadef_object.MetadefObject`
+        """
+        namespace_name = resource.Resource._get_id(namespace)
+        return self._create(
+            _metadef_object.MetadefObject,
+            namespace_name=namespace_name,
+            **attrs,
+        )
+
+    def get_metadef_object(self, metadef_object, namespace):
+        """Get a single metadef object
+
+        :param metadef_object: The value can be the ID of a metadef_object
+            or a
+            :class:`~openstack.image.v2.metadef_object.MetadefObject`
+            instance.
+        :param namespace: The value can be either the name of a metadef
+            namespace or a
+            :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            instance.
+        :returns: One :class:`~openstack.image.v2.metadef_object.MetadefObject`
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
+            resource can be found.
+        """
+        object_name = resource.Resource._get_id(metadef_object)
+        namespace_name = resource.Resource._get_id(namespace)
+        return self._get(
+            _metadef_object.MetadefObject,
+            namespace_name=namespace_name,
+            name=object_name,
+        )
+
+    def metadef_objects(self, namespace):
+        """Get metadef object list of the namespace
+
+        :param namespace: The value can be either the name of a metadef
+            namespace or a
+            :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            instance.
+
+        :returns: One :class:`~openstack.image.v2.metadef_object.MetadefObject`
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
+            resource can be found.
+        """
+        namespace_name = resource.Resource._get_id(namespace)
+        return self._list(
+            _metadef_object.MetadefObject,
+            namespace_name=namespace_name,
+        )
+
+    def update_metadef_object(self, metadef_object, namespace, **attrs):
+        """Update a single metadef object
+
+        :param metadef_object: The value can be the ID of a metadef_object or a
+            :class:`~openstack.image.v2.metadef_object.MetadefObject` instance.
+        :param namespace: The value can be either the name of a metadef
+            namespace or a
+            :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            instance.
+        :param dict attrs: Keyword arguments which will be used to update
+            a :class:`~openstack.image.v2.metadef_object.MetadefObject`
+
+        :returns: One :class:`~openstack.image.v2.metadef_object.MetadefObject`
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
+            resource can be found.
+        """
+        namespace_name = resource.Resource._get_id(namespace)
+        metadef_object = resource.Resource._get_id(metadef_object)
+        return self._update(
+            _metadef_object.MetadefObject,
+            metadef_object,
+            namespace_name=namespace_name,
+            **attrs,
+        )
+
+    def delete_metadef_object(self, metadef_object, namespace, **attrs):
+        """Removes a single metadef object
+
+        :param metadef_object: The value can be the ID of a metadef_object or a
+            :class:`~openstack.image.v2.metadef_object.MetadefObject` instance.
+        :param namespace: The value can be either the name of a metadef
+            namespace or a
+            :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            instance.
+        :param dict attrs: Keyword arguments which will be used to update
+            a :class:`~openstack.image.v2.metadef_object.MetadefObject`
+
+        :returns: ``None``
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
+            resource can be found.
+        """
+        namespace_name = resource.Resource._get_id(namespace)
+        return self._delete(
+            _metadef_object.MetadefObject,
+            metadef_object,
+            namespace_name=namespace_name,
+            **attrs,
+        )
+
+    def delete_all_metadef_objects(self, namespace):
+        """Delete all objects
+
+        :param namespace: The value can be either the name of a metadef
+            namespace or a
+            :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            instance.
+        :returns: ``None``
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
+            resource can be found.
+        """
+        namespace = self._get_resource(
+            _metadef_namespace.MetadefNamespace, namespace
+        )
+        return namespace.delete_all_objects(self)
+
     # ====== METADEF RESOURCE TYPES ======
     def metadef_resource_types(self, **query):
         """Return a generator of metadef resource types
@@ -1212,8 +1335,8 @@ class Proxy(proxy.Proxy):
         :return: A generator object of metadef resource types
         :rtype:
             :class:`~openstack.image.v2.metadef_resource_type.MetadefResourceType`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
-                when no resource can be found.
+        :raises: :class:`~openstack.exceptions.NotFoundException`
+            when no resource can be found.
         """
         return self._list(_metadef_resource_type.MetadefResourceType, **query)
 
@@ -1259,7 +1382,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
             instance
         :param bool ignore_missing: When set to ``False``,
-            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
+            :class:`~openstack.exceptions.NotFoundException` will be raised when
             the metadef resource type association does not exist.
         :returns: ``None``
         """
@@ -1281,7 +1404,7 @@ class Proxy(proxy.Proxy):
         :return: A generator object of metadef resource type associations
         :rtype:
             :class:`~openstack.image.v2.metadef_resource_type.MetadefResourceTypeAssociation`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
                 when no resource can be found.
         """
         namespace_name = resource.Resource._get_id(metadef_namespace)
@@ -1291,12 +1414,155 @@ class Proxy(proxy.Proxy):
             **query,
         )
 
+    # ====== METADEF PROPERTY ======
+    def create_metadef_property(self, metadef_namespace, **attrs):
+        """Create a metadef property
+
+        :param metadef_namespace: The value can be either the name of metadef
+            namespace or an
+            :class:`~openstack.image.v2.metadef_property.MetadefNamespace`
+            instance
+        :param attrs: The attributes to create on the metadef property
+            represented by ``metadef_property``.
+
+        :returns: The created metadef property
+        :rtype: :class:`~openstack.image.v2.metadef_property.MetadefProperty`
+        """
+        namespace_name = resource.Resource._get_id(metadef_namespace)
+        return self._create(
+            _metadef_property.MetadefProperty,
+            namespace_name=namespace_name,
+            **attrs,
+        )
+
+    def update_metadef_property(
+        self, metadef_property, metadef_namespace, **attrs
+    ):
+        """Update a metadef property
+
+        :param metadef_property: The value can be either the name of metadef
+            property or an
+            :class:`~openstack.image.v2.metadef_property.MetadefProperty`
+            instance.
+        :param metadef_namespace: The value can be either the name of metadef
+            namespace or an
+            :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            instance
+        :param attrs: The attributes to update on the metadef property
+            represented by ``metadef_property``.
+
+        :returns: The updated metadef property
+        :rtype: :class:`~openstack.image.v2.metadef_property.MetadefProperty`
+        """
+        namespace_name = resource.Resource._get_id(metadef_namespace)
+        metadef_property = resource.Resource._get_id(metadef_property)
+        return self._update(
+            _metadef_property.MetadefProperty,
+            metadef_property,
+            namespace_name=namespace_name,
+            **attrs,
+        )
+
+    def delete_metadef_property(
+        self, metadef_property, metadef_namespace, ignore_missing=True
+    ):
+        """Delete a metadef property
+
+        :param metadef_property: The value can be either the name of metadef
+            property or an
+            :class:`~openstack.image.v2.metadef_property.MetadefProperty`
+            instance
+        :param metadef_namespace: The value can be either the name of metadef
+            namespace or an
+            :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            instance
+        :param bool ignore_missing: When set to
+            ``False`` :class:`~openstack.exceptions.NotFoundException` will be
+            raised when the instance does not exist. When set to ``True``,
+            no exception will be set when attempting to delete a nonexistent
+            instance.
+
+        :returns: ``None``
+        """
+        namespace_name = resource.Resource._get_id(metadef_namespace)
+        metadef_property = resource.Resource._get_id(metadef_property)
+        return self._delete(
+            _metadef_property.MetadefProperty,
+            metadef_property,
+            namespace_name=namespace_name,
+            ignore_missing=ignore_missing,
+        )
+
+    def metadef_properties(self, metadef_namespace, **query):
+        """Return a generator of metadef properties
+
+        :param metadef_namespace: The value can be either the name of metadef
+            namespace or an
+            :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            instance
+        :param kwargs query: Optional query parameters to be sent to limit
+            the resources being returned.
+
+        :returns: A generator of property objects
+        """
+        namespace_name = resource.Resource._get_id(metadef_namespace)
+        return self._list(
+            _metadef_property.MetadefProperty,
+            requires_id=False,
+            namespace_name=namespace_name,
+            **query,
+        )
+
+    def get_metadef_property(
+        self, metadef_property, metadef_namespace, **query
+    ):
+        """Get a single metadef property
+
+        :param metadef_property: The value can be either the name of metadef
+            property or an
+            :class:`~openstack.image.v2.metadef_property.MetadefProperty`
+            instance.
+        :param metadef_namespace: The value can be either the name of metadef
+            namespace or an
+            :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            instance
+
+        :returns: One
+            :class:`~~openstack.image.v2.metadef_property.MetadefProperty`
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
+            resource can be found.
+        """
+        namespace_name = resource.Resource._get_id(metadef_namespace)
+        return self._get(
+            _metadef_property.MetadefProperty,
+            metadef_property,
+            namespace_name=namespace_name,
+            **query,
+        )
+
+    def delete_all_metadef_properties(self, metadef_namespace):
+        """Delete all metadata definitions property inside a specific namespace.
+
+        :param metadef_namespace: The value can be either the name of a metadef
+            namespace or a
+            :class:`~openstack.image.v2.metadef_namespace.MetadefNamespace`
+            instance.
+
+        :returns: ``None``
+        :raises: :class:`~openstack.exceptions.NotFoundException` when no
+            resource can be found.
+        """
+        namespace = self._get_resource(
+            _metadef_namespace.MetadefNamespace, metadef_namespace
+        )
+        return namespace.delete_all_properties(self)
+
     # ====== SCHEMAS ======
     def get_images_schema(self):
         """Get images schema
 
         :returns: One :class:`~openstack.image.v2.schema.Schema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(
@@ -1309,7 +1575,7 @@ class Proxy(proxy.Proxy):
         """Get single image schema
 
         :returns: One :class:`~openstack.image.v2.schema.Schema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(
@@ -1322,7 +1588,7 @@ class Proxy(proxy.Proxy):
         """Get image members schema
 
         :returns: One :class:`~openstack.image.v2.schema.Schema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(
@@ -1335,7 +1601,7 @@ class Proxy(proxy.Proxy):
         """Get image member schema
 
         :returns: One :class:`~openstack.image.v2.schema.Schema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(
@@ -1348,7 +1614,7 @@ class Proxy(proxy.Proxy):
         """Get image tasks schema
 
         :returns: One :class:`~openstack.image.v2.schema.Schema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(
@@ -1361,7 +1627,7 @@ class Proxy(proxy.Proxy):
         """Get image task schema
 
         :returns: One :class:`~openstack.image.v2.schema.Schema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(
@@ -1374,7 +1640,7 @@ class Proxy(proxy.Proxy):
         """Get metadata definition namespace schema
 
         :returns: One :class:`~openstack.image.v2.metadef_schema.MetadefSchema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(
@@ -1387,7 +1653,7 @@ class Proxy(proxy.Proxy):
         """Get metadata definition namespaces schema
 
         :returns: One :class:`~openstack.image.v2.metadef_schema.MetadefSchema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(
@@ -1400,7 +1666,7 @@ class Proxy(proxy.Proxy):
         """Get metadata definition resource type association schema
 
         :returns: One :class:`~openstack.image.v2.metadef_schema.MetadefSchema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(
@@ -1413,7 +1679,7 @@ class Proxy(proxy.Proxy):
         """Get metadata definition resource type associations schema
 
         :returns: One :class:`~openstack.image.v2.metadef_schema.MetadefSchema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(
@@ -1426,7 +1692,7 @@ class Proxy(proxy.Proxy):
         """Get metadata definition object schema
 
         :returns: One :class:`~openstack.image.v2.metadef_schema.MetadefSchema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(
@@ -1439,7 +1705,7 @@ class Proxy(proxy.Proxy):
         """Get metadata definition objects schema
 
         :returns: One :class:`~openstack.image.v2.metadef_schema.MetadefSchema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(
@@ -1452,7 +1718,7 @@ class Proxy(proxy.Proxy):
         """Get metadata definition property schema
 
         :returns: One :class:`~openstack.image.v2.metadef_schema.MetadefSchema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(
@@ -1465,7 +1731,7 @@ class Proxy(proxy.Proxy):
         """Get metadata definition properties schema
 
         :returns: One :class:`~openstack.image.v2.metadef_schema.MetadefSchema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(
@@ -1478,7 +1744,7 @@ class Proxy(proxy.Proxy):
         """Get metadata definition tag schema
 
         :returns: One :class:`~openstack.image.v2.metadef_schema.MetadefSchema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(
@@ -1491,7 +1757,7 @@ class Proxy(proxy.Proxy):
         """Get metadata definition tags schema
 
         :returns: One :class:`~openstack.image.v2.metadef_schema.MetadefSchema`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(
@@ -1519,7 +1785,7 @@ class Proxy(proxy.Proxy):
             :class:`~openstack.image.v2.task.Task` instance.
 
         :returns: One :class:`~openstack.image.v2.task.Task`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(_task.Task, task)
@@ -1572,7 +1838,7 @@ class Proxy(proxy.Proxy):
         if task.status.lower() == status.lower():
             return task
 
-        name = "{res}:{id}".format(res=task.__class__.__name__, id=task.id)
+        name = f"{task.__class__.__name__}:{task.id}"
         msg = "Timeout waiting for {name} to transition to {status}".format(
             name=name, status=status
         )
@@ -1621,7 +1887,7 @@ class Proxy(proxy.Proxy):
         :rtype: :class:`~openstack.image.v2.service_info.Store`
         """
         if details:
-            query['base_path'] = utils.urljoin(_si.Store, 'details')
+            query['base_path'] = utils.urljoin(_si.Store.base_path, 'detail')
         return self._list(_si.Store, **query)
 
     # ====== IMPORTS ======
@@ -1629,7 +1895,7 @@ class Proxy(proxy.Proxy):
         """Get a info about image constraints
 
         :returns: One :class:`~openstack.image.v2.service_info.Import`
-        :raises: :class:`~openstack.exceptions.ResourceNotFound`
+        :raises: :class:`~openstack.exceptions.NotFoundException`
             when no resource can be found.
         """
         return self._get(_si.Import, requires_id=False)
@@ -1649,3 +1915,32 @@ class Proxy(proxy.Proxy):
             to delete failed to occur in the specified seconds.
         """
         return resource.wait_for_delete(self, res, interval, wait)
+
+    def _get_cleanup_dependencies(self):
+        return {'image': {'before': ['identity']}}
+
+    def _service_cleanup(
+        self,
+        dry_run=True,
+        client_status_queue=None,
+        identified_resources=None,
+        filters=None,
+        resource_evaluation_fn=None,
+        skip_resources=None,
+    ):
+        if self.should_skip_resource_cleanup("image", skip_resources):
+            return
+
+        project_id = self.get_project_id()
+
+        # Note that images cannot be deleted when they are still being used
+        for obj in self.images(owner=project_id):
+            self._service_cleanup_del_res(
+                self.delete_image,
+                obj,
+                dry_run=dry_run,
+                client_status_queue=client_status_queue,
+                identified_resources=identified_resources,
+                filters=filters,
+                resource_evaluation_fn=resource_evaluation_fn,
+            )
