@@ -30,18 +30,25 @@ class Router(_base.NetworkResource, tag.TagMixin):
 
     # NOTE: We don't support query on datetime, list or dict fields
     _query_mapping = resource.QueryParameters(
-        'description', 'flavor_id', 'name', 'status', 'project_id',
+        'description',
+        'flavor_id',
+        'name',
+        'status',
+        'project_id',
+        'sort_key',
+        'sort_dir',
         is_admin_state_up='admin_state_up',
         is_distributed='distributed',
         is_ha='ha',
-        **tag.TagMixin._tag_query_parameters
+        **tag.TagMixin._tag_query_parameters,
     )
 
     # Properties
     #: Availability zone hints to use when scheduling the router.
     #: *Type: list of availability zone names*
-    availability_zone_hints = resource.Body('availability_zone_hints',
-                                            type=list)
+    availability_zone_hints = resource.Body(
+        'availability_zone_hints', type=list
+    )
     #: Availability zones for the router.
     #: *Type: list of availability zone names*
     availability_zones = resource.Body('availability_zones', type=list)
@@ -49,6 +56,8 @@ class Router(_base.NetworkResource, tag.TagMixin):
     created_at = resource.Body('created_at')
     #: The router description.
     description = resource.Body('description')
+    #: The ndp proxy state of the router
+    enable_ndp_proxy = resource.Body('enable_ndp_proxy', type=bool)
     #: The ``network_id``, for the external gateway. *Type: dict*
     external_gateway_info = resource.Body('external_gateway_info', type=dict)
     #: The ID of the flavor.
@@ -112,7 +121,7 @@ class Router(_base.NetworkResource, tag.TagMixin):
         resp = self._put(session, url, body)
         return resp.json()
 
-    def add_extra_routes(self, session, body):
+    def add_extra_routes(self, session, body) -> 'Router':
         """Add extra routes to a logical router.
 
         :param session: The session to communicate through.
@@ -128,7 +137,7 @@ class Router(_base.NetworkResource, tag.TagMixin):
         self._translate_response(resp)
         return self
 
-    def remove_extra_routes(self, session, body):
+    def remove_extra_routes(self, session, body) -> 'Router':
         """Remove extra routes from a logical router.
 
         :param session: The session to communicate through.
@@ -153,8 +162,7 @@ class Router(_base.NetworkResource, tag.TagMixin):
 
         :returns: The body of the response as a dictionary.
         """
-        url = utils.urljoin(self.base_path, self.id,
-                            'add_gateway_router')
+        url = utils.urljoin(self.base_path, self.id, 'add_gateway_router')
         resp = session.put(url, json=body)
         return resp.json()
 
@@ -167,11 +175,56 @@ class Router(_base.NetworkResource, tag.TagMixin):
 
         :returns: The body of the response as a dictionary.
         """
-        url = utils.urljoin(self.base_path, self.id,
-                            'remove_gateway_router')
+        url = utils.urljoin(self.base_path, self.id, 'remove_gateway_router')
         resp = session.put(url, json=body)
         return resp.json()
 
+    def add_external_gateways(self, session, body):
+        """Add external gateways to a router.
+
+        :param session: The session to communicate through.
+        :type session: :class:`~keystoneauth1.adapter.Adapter`
+        :param dict body: The body requested to be updated on the router
+
+        :returns: The body of the response as a dictionary.
+        """
+        url = utils.urljoin(self.base_path, self.id, 'add_external_gateways')
+        resp = session.put(url, json=body)
+        self._translate_response(resp)
+        return self
+
+    def update_external_gateways(self, session, body):
+        """Update external gateways of a router.
+
+        :param session: The session to communicate through.
+        :type session: :class:`~keystoneauth1.adapter.Adapter`
+        :param dict body: The body requested to be updated on the router
+
+        :returns: The body of the response as a dictionary.
+        """
+        url = utils.urljoin(
+            self.base_path, self.id, 'update_external_gateways'
+        )
+        resp = session.put(url, json=body)
+        self._translate_response(resp)
+        return self
+
+    def remove_external_gateways(self, session, body):
+        """Remove external gateways from a router.
+
+        :param session: The session to communicate through.
+        :type session: :class:`~keystoneauth1.adapter.Adapter`
+        :param dict body: The body requested to be updated on the router
+
+        :returns: The body of the response as a dictionary.
+        """
+        url = utils.urljoin(
+            self.base_path, self.id, 'remove_external_gateways'
+        )
+        resp = session.put(url, json=body)
+        self._translate_response(resp)
+        return self
+
 
 class L3AgentRouter(Router):
     resource_key = 'router'
@@ -186,4 +239,5 @@ class L3AgentRouter(Router):
     allow_delete = False
     allow_list = True
 
+
 # NOTE: No query parameter is supported
