@@ -36,7 +36,7 @@ class Stack(resource.Resource):
         'owner_id',
         'username',
         project_id='tenant_id',
-        **tag.TagMixin._tag_query_parameters
+        **tag.TagMixin._tag_query_parameters,
     )
 
     # Properties
@@ -112,35 +112,37 @@ class Stack(resource.Resource):
     #: The ID of the user project created for this stack.
     user_project_id = resource.Body('stack_user_project_id')
 
-    def create(self, session, base_path=None):
+    def create(self, session, prepend_key=False, *args, **kwargs):
         # This overrides the default behavior of resource creation because
         # heat doesn't accept resource_key in its request.
-        return super(Stack, self).create(
-            session, prepend_key=False, base_path=base_path
-        )
+        return super().create(session, prepend_key, *args, **kwargs)
 
-    def commit(self, session, base_path=None):
-        # This overrides the default behavior of resource creation because
-        # heat doesn't accept resource_key in its request.
-        return super(Stack, self).commit(
-            session, prepend_key=False, has_body=False, base_path=None
-        )
-
-    def update(self, session, preview=False):
+    def commit(
+        self,
+        session,
+        prepend_key=True,
+        has_body=True,
+        retry_on_conflict=None,
+        base_path=None,
+        *,
+        microversion=None,
+        preview=False,
+        **kwargs,
+    ):
         # This overrides the default behavior of resource update because
         # we need to use other endpoint for update preview.
         base_path = None
         if self.name and self.id:
-            base_path = '/stacks/%(stack_name)s/%(stack_id)s' % {
-                'stack_name': self.name,
-                'stack_id': self.id,
-            }
+            base_path = '/stacks/{stack_name}/{stack_id}'.format(
+                stack_name=self.name,
+                stack_id=self.id,
+            )
         elif self.name or self.id:
             # We have only one of name/id. Do not try to build a stacks/NAME/ID
             # path
-            base_path = '/stacks/%(stack_identity)s' % {
-                'stack_identity': self.name or self.id
-            }
+            base_path = '/stacks/{stack_identity}'.format(
+                stack_identity=self.name or self.id
+            )
         request = self._prepare_request(
             prepend_key=False, requires_id=False, base_path=base_path
         )
@@ -165,8 +167,9 @@ class Stack(resource.Resource):
     def _action(self, session, body):
         """Perform stack actions"""
         url = utils.urljoin(self.base_path, self._get_id(self), 'actions')
-        resp = session.post(url, json=body)
-        return resp.json()
+        resp = session.post(url, json=body, microversion=self.microversion)
+        exceptions.raise_from_response(resp)
+        return resp
 
     def check(self, session):
         return self._action(session, {'check': ''})
@@ -178,6 +181,37 @@ class Stack(resource.Resource):
         resp = session.delete(url)
         return resp.json()
 
+    def export(self, session):
+        """Export a stack data
+
+        :param session: The session to use for making this request.
+        :return: A dictionary containing the stack data.
+        """
+        url = utils.urljoin(
+            self.base_path, self.name, self._get_id(self), 'export'
+        )
+        resp = session.get(url)
+        exceptions.raise_from_response(resp)
+        return resp.json()
+
+    def suspend(self, session):
+        """Suspend a stack
+
+        :param session: The session to use for making this request
+        :returns: None
+        """
+        body = {"suspend": None}
+        self._action(session, body)
+
+    def resume(self, session):
+        """Resume a stack
+
+        :param session: The session to use for making this request
+        :returns: None
+        """
+        body = {"resume": None}
+        self._action(session, body)
+
     def fetch(
         self,
         session,
@@ -185,7 +219,9 @@ class Stack(resource.Resource):
         base_path=None,
         error_message=None,
         skip_cache=False,
+        *,
         resolve_outputs=True,
+        **params,
     ):
         if not self.allow_fetch:
             raise exceptions.MethodNotSupported(self, "fetch")
@@ -212,7 +248,7 @@ class Stack(resource.Resource):
         self._translate_response(response, **kwargs)
 
         if self and self.status in ['DELETE_COMPLETE', 'ADOPT_COMPLETE']:
-            raise exceptions.ResourceNotFound(
+            raise exceptions.NotFoundException(
                 "No stack found for %s" % self.id
             )
         return self
@@ -226,7 +262,7 @@ class Stack(resource.Resource):
         :param name_or_id: This resource's identifier, if needed by
                            the request. The default is ``None``.
         :param bool ignore_missing: When set to ``False``
-                    :class:`~openstack.exceptions.ResourceNotFound` will be
+                    :class:`~openstack.exceptions.NotFoundException` will be
                     raised when the resource does not exist.
                     When set to ``True``, None will be returned when
                     attempting to find a nonexistent resource.
@@ -239,7 +275,7 @@ class Stack(resource.Resource):
                  or None if nothing matches.
         :raises: :class:`openstack.exceptions.DuplicateResource` if more
                  than one resource is found for this request.
-        :raises: :class:`openstack.exceptions.ResourceNotFound` if nothing
+        :raises: :class:`openstack.exceptions.NotFoundException` if nothing
                  is found and ignore_missing is ``False``.
         """
         session = cls._get_session(session)
@@ -257,8 +293,8 @@ class Stack(resource.Resource):
 
         if ignore_missing:
             return None
-        raise exceptions.ResourceNotFound(
-            "No %s found for %s" % (cls.__name__, name_or_id)
+        raise exceptions.NotFoundException(
+            f"No {cls.__name__} found for {name_or_id}"
         )
 
 
