@@ -11,20 +11,17 @@
 # under the License.
 
 import functools
-from typing import Generator
-from typing import Generic
-from typing import Optional
-from typing import Type
-from typing import TypeVar
+import typing as ty
 import urllib
 from urllib.parse import urlparse
+import warnings
 
 try:
     import simplejson
 
     JSONDecodeError = simplejson.scanner.JSONDecodeError
 except ImportError:
-    JSONDecodeError = ValueError
+    JSONDecodeError = ValueError  # type: ignore
 import iso8601
 import jmespath
 from keystoneauth1 import adapter
@@ -32,8 +29,10 @@ from keystoneauth1 import adapter
 from openstack import _log
 from openstack import exceptions
 from openstack import resource
+from openstack import warnings as os_warnings
 
-T = TypeVar('T')
+
+ResourceType = ty.TypeVar('ResourceType', bound=resource.Resource)
 
 
 # The _check_resource decorator is used on Proxy methods to ensure that
@@ -74,17 +73,17 @@ def normalize_metric_name(name):
     return name
 
 
-class Proxy(adapter.Adapter, Generic[T]):
+class Proxy(adapter.Adapter):
     """Represents a service."""
 
-    retriable_status_codes = None
+    retriable_status_codes: ty.Optional[ty.List[int]] = None
     """HTTP status codes that should be retried by default.
 
     The number of retries is defined by the configuration in parameters called
     ``<service-type>_status_code_retries``.
     """
 
-    _resource_registry = dict()
+    _resource_registry: ty.Dict[str, ty.Type[resource.Resource]] = {}
     """Registry of the supported resourses.
 
     Dictionary of resource names (key) types (value).
@@ -115,7 +114,7 @@ class Proxy(adapter.Adapter, Generic[T]):
         self._influxdb_client = influxdb_client
         self._influxdb_config = influxdb_config
         if self.service_type:
-            log_name = 'openstack.{0}'.format(self.service_type)
+            log_name = f'openstack.{self.service_type}'
         else:
             log_name = 'openstack'
         self.log = _log.setup_logging(log_name)
@@ -227,6 +226,12 @@ class Proxy(adapter.Adapter, Generic[T]):
           /servers/{id}/os-security-groups -> ['server', 'os-security-groups']
           /v2.0/networks.json -> ['networks']
         """
+        if service_type is not None:
+            warnings.warn(
+                "The 'service_type' parameter is unnecesary and will be "
+                "removed in a future release.",
+                os_warnings.RemovedInSDK60Warning,
+            )
 
         url_path = urllib.parse.urlparse(url).path.strip()
         # Remove / from the beginning to keep the list indexes of interesting
@@ -335,7 +340,9 @@ class Proxy(adapter.Adapter, Generic[T]):
             with self._statsd_client.pipeline() as pipe:
                 if response is not None:
                     duration = int(response.elapsed.total_seconds() * 1000)
-                    metric_name = '%s.%s' % (key, str(response.status_code))
+                    metric_name = '{}.{}'.format(
+                        key, str(response.status_code)
+                    )
                     pipe.timing(metric_name, duration)
                     pipe.incr(metric_name)
                     if duration > 1000:
@@ -398,7 +405,7 @@ class Proxy(adapter.Adapter, Generic[T]):
             tags['status_code'] = str(response.status_code)
             # Note(gtema): emit also status_code as a value (counter)
             fields[str(response.status_code)] = 1
-            fields['%s.%s' % (method, response.status_code)] = 1
+            fields[f'{method}.{response.status_code}'] = 1
             # Note(gtema): status_code field itself is also very helpful on the
             # graphs to show what was the code, instead of counting its
             # occurences
@@ -413,19 +420,13 @@ class Proxy(adapter.Adapter, Generic[T]):
             else 'openstack_api'
         )
         # Note(gtema) append service name into the measurement name
-        measurement = '%s.%s' % (measurement, self.service_type)
+        measurement = f'{measurement}.{self.service_type}'
         data = [dict(measurement=measurement, tags=tags, fields=fields)]
         try:
             self._influxdb_client.write_points(data)
         except Exception:
             self.log.exception('Error writing statistics to InfluxDB')
 
-    def _version_matches(self, version):
-        api_version = self.get_api_major_version()
-        if api_version:
-            return api_version[0] == version
-        return False
-
     def _get_connection(self):
         """Get the Connection object associated with this Proxy.
 
@@ -437,7 +438,9 @@ class Proxy(adapter.Adapter, Generic[T]):
             self, '_connection', getattr(self.session, '_sdk_connection', None)
         )
 
-    def _get_resource(self, resource_type: Type[T], value, **attrs) -> T:
+    def _get_resource(
+        self, resource_type: ty.Type[ResourceType], value, **attrs
+    ) -> ResourceType:
         """Get a resource object to work on
 
         :param resource_type: The type of resource to operate on. This should
@@ -483,14 +486,47 @@ class Proxy(adapter.Adapter, Generic[T]):
             value = resource.Resource._get_id(parent)
         return value
 
+    @ty.overload
     def _find(
-        self, resource_type: Type[T], name_or_id, ignore_missing=True, **attrs
-    ) -> Optional[T]:
+        self,
+        resource_type: ty.Type[ResourceType],
+        name_or_id: str,
+        ignore_missing: ty.Literal[True] = True,
+        **attrs,
+    ) -> ty.Optional[ResourceType]: ...
+
+    @ty.overload
+    def _find(
+        self,
+        resource_type: ty.Type[ResourceType],
+        name_or_id: str,
+        ignore_missing: ty.Literal[False],
+        **attrs,
+    ) -> ResourceType: ...
+
+    # excuse the duplication here: it's mypy's fault
+    # https://github.com/python/mypy/issues/14764
+    @ty.overload
+    def _find(
+        self,
+        resource_type: ty.Type[ResourceType],
+        name_or_id: str,
+        ignore_missing: bool,
+        **attrs,
+    ) -> ty.Optional[ResourceType]: ...
+
+    def _find(
+        self,
+        resource_type: ty.Type[ResourceType],
+        name_or_id: str,
+        ignore_missing: bool = True,
+        **attrs,
+    ) -> ty.Optional[ResourceType]:
         """Find a resource
 
         :param name_or_id: The name or ID of a resource to find.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the resource does not exist.
             When set to ``True``, None will be returned when
             attempting to find a nonexistent resource.
@@ -506,8 +542,12 @@ class Proxy(adapter.Adapter, Generic[T]):
 
     @_check_resource(strict=False)
     def _delete(
-        self, resource_type: Type[T], value, ignore_missing=True, **attrs
-    ):
+        self,
+        resource_type: ty.Type[ResourceType],
+        value,
+        ignore_missing=True,
+        **attrs,
+    ) -> ty.Optional[ResourceType]:
         """Delete a resource
 
         :param resource_type: The type of resource to delete. This should
@@ -517,7 +557,7 @@ class Proxy(adapter.Adapter, Generic[T]):
             resource or a :class:`~openstack.resource.Resource`
             subclass.
         :param bool ignore_missing: When set to ``False``
-            :class:`~openstack.exceptions.ResourceNotFound` will be
+            :class:`~openstack.exceptions.NotFoundException` will be
             raised when the resource does not exist.
             When set to ``True``, no exception will be set when
             attempting to delete a nonexistent resource.
@@ -527,7 +567,7 @@ class Proxy(adapter.Adapter, Generic[T]):
         :raises: ``ValueError`` if ``value`` is a
             :class:`~openstack.resource.Resource` that doesn't match
             the ``resource_type``.
-            :class:`~openstack.exceptions.ResourceNotFound` when
+            :class:`~openstack.exceptions.NotFoundException` when
             ignore_missing if ``False`` and a nonexistent resource
             is attempted to be deleted.
         """
@@ -535,7 +575,7 @@ class Proxy(adapter.Adapter, Generic[T]):
 
         try:
             rv = res.delete(self)
-        except exceptions.ResourceNotFound:
+        except exceptions.NotFoundException:
             if ignore_missing:
                 return None
             raise
@@ -544,8 +584,12 @@ class Proxy(adapter.Adapter, Generic[T]):
 
     @_check_resource(strict=False)
     def _update(
-        self, resource_type: Type[T], value, base_path=None, **attrs
-    ) -> T:
+        self,
+        resource_type: ty.Type[ResourceType],
+        value,
+        base_path=None,
+        **attrs,
+    ) -> ResourceType:
         """Update a resource
 
         :param resource_type: The type of resource to update.
@@ -569,7 +613,12 @@ class Proxy(adapter.Adapter, Generic[T]):
         res = self._get_resource(resource_type, value, **attrs)
         return res.commit(self, base_path=base_path)
 
-    def _create(self, resource_type: Type[T], base_path=None, **attrs):
+    def _create(
+        self,
+        resource_type: ty.Type[ResourceType],
+        base_path=None,
+        **attrs,
+    ) -> ResourceType:
         """Create a resource from attributes
 
         :param resource_type: The type of resource to create.
@@ -589,13 +638,23 @@ class Proxy(adapter.Adapter, Generic[T]):
         :returns: The result of the ``create``
         :rtype: :class:`~openstack.resource.Resource`
         """
+        # Check for attributes whose names conflict with the parameters
+        # specified in the method.
+        conflicting_attrs = attrs.get('__conflicting_attrs', {})
+        if conflicting_attrs:
+            for k, v in conflicting_attrs.items():
+                attrs[k] = v
+            attrs.pop('__conflicting_attrs')
         conn = self._get_connection()
         res = resource_type.new(connection=conn, **attrs)
         return res.create(self, base_path=base_path)
 
     def _bulk_create(
-        self, resource_type: Type[T], data, base_path=None
-    ) -> Generator[T, None, None]:
+        self,
+        resource_type: ty.Type[ResourceType],
+        data,
+        base_path=None,
+    ) -> ty.Generator[ResourceType, None, None]:
         """Create a resource from attributes
 
         :param resource_type: The type of resource to create.
@@ -618,13 +677,13 @@ class Proxy(adapter.Adapter, Generic[T]):
     @_check_resource(strict=False)
     def _get(
         self,
-        resource_type: Type[T],
+        resource_type: ty.Type[ResourceType],
         value=None,
         requires_id=True,
         base_path=None,
         skip_cache=False,
         **attrs,
-    ):
+    ) -> ResourceType:
         """Fetch a resource
 
         :param resource_type: The type of resource to get.
@@ -661,12 +720,12 @@ class Proxy(adapter.Adapter, Generic[T]):
 
     def _list(
         self,
-        resource_type: Type[T],
+        resource_type: ty.Type[ResourceType],
         paginated=True,
         base_path=None,
         jmespath_filters=None,
         **attrs,
-    ) -> Generator[T, None, None]:
+    ) -> ty.Generator[ResourceType, None, None]:
         """List a resource
 
         :param resource_type: The type of resource to list. This should
@@ -692,6 +751,14 @@ class Proxy(adapter.Adapter, Generic[T]):
             :class:`~openstack.resource.Resource` that doesn't match
             the ``resource_type``.
         """
+        # Check for attributes whose names conflict with the parameters
+        # specified in the method.
+        conflicting_attrs = attrs.get('__conflicting_attrs', {})
+        if conflicting_attrs:
+            for k, v in conflicting_attrs.items():
+                attrs[k] = v
+            attrs.pop('__conflicting_attrs')
+
         data = resource_type.list(
             self, paginated=paginated, base_path=base_path, **attrs
         )
@@ -702,8 +769,12 @@ class Proxy(adapter.Adapter, Generic[T]):
         return data
 
     def _head(
-        self, resource_type: Type[T], value=None, base_path=None, **attrs
-    ):
+        self,
+        resource_type: ty.Type[ResourceType],
+        value=None,
+        base_path=None,
+        **attrs,
+    ) -> ResourceType:
         """Retrieve a resource's header
 
         :param resource_type: The type of resource to retrieve.
@@ -823,7 +894,7 @@ class Proxy(adapter.Adapter, Generic[T]):
 
         if resource_name in skip_resources:
             self.log.debug(
-                f"Skipping resource {resource_name} " "in project cleanup"
+                f"Skipping resource {resource_name} in project cleanup"
             )
             return True
 
