@@ -10,6 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import copy
 from unittest import mock
 
 from keystoneauth1 import adapter
@@ -22,14 +23,15 @@ FAKE_ID = "6685584b-1eac-4da6-b5c3-555430cf68ff"
 IMAGE_METADATA = {
     'container_format': 'bare',
     'min_ram': '64',
-    'disk_format': u'qcow2',
+    'disk_format': 'qcow2',
     'image_name': 'TestVM',
     'image_id': '625d4f2c-cf67-4af3-afb6-c7220f766947',
     'checksum': '64d7c1cd2b6f60c92c14662941cb7913',
     'min_disk': '0',
-    u'size': '13167616',
+    'size': '13167616',
 }
 
+FAKE_HOST = "fake_host@fake_backend#fake_pool"
 VOLUME = {
     "status": "creating",
     "name": "my_volume",
@@ -56,7 +58,6 @@ VOLUME = {
     "os-volume-replication:extended_status": "really nah",
     "consistencygroup_id": "123asf-asdf123",
     "os-volume-replication:driver_data": "ahasadfasdfasdfasdfsdf",
-    "snapshot_id": "93c2e2aa-7744-4fd6-a31a-80c4726b08d7",
     "encrypted": "false",
     "OS-SCH-HNT:scheduler_hints": {
         "same_host": [
@@ -143,13 +144,13 @@ class TestVolume(base.TestCase):
 
 class TestVolumeActions(TestVolume):
     def setUp(self):
-        super(TestVolumeActions, self).setUp()
+        super().setUp()
         self.resp = mock.Mock()
         self.resp.body = None
         self.resp.status_code = 200
         self.resp.json = mock.Mock(return_value=self.resp.body)
         self.sess = mock.Mock(spec=adapter.Adapter)
-        self.sess.default_microversion = '3.0'
+        self.sess.default_microversion = '3.60'
         self.sess.post = mock.Mock(return_value=self.resp)
         self.sess._get_connection = mock.Mock(return_value=self.cloud)
 
@@ -208,6 +209,52 @@ class TestVolumeActions(TestVolume):
             url, json=body, microversion=sot._max_microversion
         )
 
+    def test_set_image_metadata(self):
+        sot = volume.Volume(**VOLUME)
+
+        self.assertIsNone(sot.set_image_metadata(self.sess, {'foo': 'bar'}))
+
+        url = 'volumes/%s/action' % FAKE_ID
+        body = {'os-set_image_metadata': {'foo': 'bar'}}
+        self.sess.post.assert_called_with(
+            url, json=body, microversion=sot._max_microversion
+        )
+
+    def test_delete_image_metadata(self):
+        _volume = copy.deepcopy(VOLUME)
+        _volume['metadata'] = {
+            'foo': 'bar',
+            'baz': 'wow',
+        }
+        sot = volume.Volume(**_volume)
+
+        self.assertIsNone(sot.delete_image_metadata(self.sess))
+
+        url = 'volumes/%s/action' % FAKE_ID
+        body_a = {'os-unset_image_metadata': 'foo'}
+        body_b = {'os-unset_image_metadata': 'baz'}
+        self.sess.post.assert_has_calls(
+            [
+                mock.call(
+                    url, json=body_a, microversion=sot._max_microversion
+                ),
+                mock.call(
+                    url, json=body_b, microversion=sot._max_microversion
+                ),
+            ]
+        )
+
+    def test_delete_image_metadata_item(self):
+        sot = volume.Volume(**VOLUME)
+
+        self.assertIsNone(sot.delete_image_metadata_item(self.sess, 'foo'))
+
+        url = 'volumes/%s/action' % FAKE_ID
+        body = {'os-unset_image_metadata': 'foo'}
+        self.sess.post.assert_called_with(
+            url, json=body, microversion=sot._max_microversion
+        )
+
     def test_reset_status(self):
         sot = volume.Volume(**VOLUME)
 
@@ -225,6 +272,21 @@ class TestVolumeActions(TestVolume):
             url, json=body, microversion=sot._max_microversion
         )
 
+    def test_reset_status__single_option(self):
+        sot = volume.Volume(**VOLUME)
+
+        self.assertIsNone(sot.reset_status(self.sess, status='1'))
+
+        url = 'volumes/%s/action' % FAKE_ID
+        body = {
+            'os-reset_status': {
+                'status': '1',
+            }
+        }
+        self.sess.post.assert_called_with(
+            url, json=body, microversion=sot._max_microversion
+        )
+
     @mock.patch(
         'openstack.utils.require_microversion',
         autospec=True,
@@ -313,7 +375,7 @@ class TestVolumeActions(TestVolume):
         self.assertIsNone(sot.unmanage(self.sess))
 
         url = 'volumes/%s/action' % FAKE_ID
-        body = {'os-unmanage': {}}
+        body = {'os-unmanage': None}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
         )
@@ -433,7 +495,7 @@ class TestVolumeActions(TestVolume):
         self.assertIsNone(sot.force_delete(self.sess))
 
         url = 'volumes/%s/action' % FAKE_ID
-        body = {'os-force_delete': {}}
+        body = {'os-force_delete': None}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
         )
@@ -503,7 +565,7 @@ class TestVolumeActions(TestVolume):
         self.assertIsNone(sot.reserve(self.sess))
 
         url = 'volumes/%s/action' % FAKE_ID
-        body = {'os-reserve': {}}
+        body = {'os-reserve': None}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
         )
@@ -514,7 +576,7 @@ class TestVolumeActions(TestVolume):
         self.assertIsNone(sot.unreserve(self.sess))
 
         url = 'volumes/%s/action' % FAKE_ID
-        body = {'os-unreserve': {}}
+        body = {'os-unreserve': None}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
         )
@@ -525,7 +587,7 @@ class TestVolumeActions(TestVolume):
         self.assertIsNone(sot.begin_detaching(self.sess))
 
         url = 'volumes/%s/action' % FAKE_ID
-        body = {'os-begin_detaching': {}}
+        body = {'os-begin_detaching': None}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
         )
@@ -536,7 +598,7 @@ class TestVolumeActions(TestVolume):
         self.assertIsNone(sot.abort_detaching(self.sess))
 
         url = 'volumes/%s/action' % FAKE_ID
-        body = {'os-roll_detaching': {}}
+        body = {'os-roll_detaching': None}
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
         )
@@ -544,7 +606,14 @@ class TestVolumeActions(TestVolume):
     def test_init_attachment(self):
         sot = volume.Volume(**VOLUME)
 
-        self.assertIsNone(sot.init_attachment(self.sess, {'a': 'b'}))
+        self.resp = mock.Mock()
+        self.resp.body = {'connection_info': {'c': 'd'}}
+        self.resp.status_code = 200
+        self.resp.json = mock.Mock(return_value=self.resp.body)
+        self.sess.post = mock.Mock(return_value=self.resp)
+        self.assertEqual(
+            {'c': 'd'}, sot.init_attachment(self.sess, {'a': 'b'})
+        )
 
         url = 'volumes/%s/action' % FAKE_ID
         body = {'os-initialize_connection': {'connector': {'a': 'b'}}}
@@ -562,3 +631,109 @@ class TestVolumeActions(TestVolume):
         self.sess.post.assert_called_with(
             url, json=body, microversion=sot._max_microversion
         )
+
+    def test__prepare_request_body(self):
+        sot = volume.Volume(**VOLUME)
+        body = sot._prepare_request_body(patch=False, prepend_key=True)
+        original_body = copy.deepcopy(sot._body.dirty)
+        # Verify that scheduler hints aren't modified after preparing request
+        # but also not part of 'volume' JSON object
+        self.assertEqual(
+            original_body['OS-SCH-HNT:scheduler_hints'],
+            body['OS-SCH-HNT:scheduler_hints'],
+        )
+        # Pop scheduler hints to verify other parameters in body
+        original_body.pop('OS-SCH-HNT:scheduler_hints')
+        # Verify that other request parameters are same but in 'volume' JSON
+        self.assertEqual(original_body, body['volume'])
+
+    def test_create_scheduler_hints(self):
+        sot = volume.Volume(**VOLUME)
+        sot._translate_response = mock.Mock()
+        sot.create(self.sess)
+
+        url = '/volumes'
+        volume_body = copy.deepcopy(VOLUME)
+        scheduler_hints = volume_body.pop('OS-SCH-HNT:scheduler_hints')
+        body = {
+            "volume": volume_body,
+            'OS-SCH-HNT:scheduler_hints': scheduler_hints,
+        }
+        self.sess.post.assert_called_with(
+            url,
+            json=body,
+            microversion='3.60',
+            headers={},
+            params={},
+        )
+
+    @mock.patch(
+        'openstack.utils.supports_microversion',
+        autospec=True,
+        return_value=True,
+    )
+    def test_manage(self, mock_mv):
+        resp = mock.Mock()
+        resp.body = {'volume': copy.deepcopy(VOLUME)}
+        resp.json = mock.Mock(return_value=resp.body)
+        resp.headers = {}
+        resp.status_code = 202
+        self.sess.post = mock.Mock(return_value=resp)
+        sot = volume.Volume.manage(self.sess, host=FAKE_HOST, ref=FAKE_ID)
+        self.assertIsNotNone(sot)
+        url = '/manageable_volumes'
+        body = {
+            'volume': {
+                'host': FAKE_HOST,
+                'ref': FAKE_ID,
+                'name': None,
+                'description': None,
+                'volume_type': None,
+                'availability_zone': None,
+                'metadata': None,
+                'bootable': False,
+            }
+        }
+        self.sess.post.assert_called_with(
+            url, json=body, microversion=sot._max_microversion
+        )
+
+    @mock.patch(
+        'openstack.utils.supports_microversion',
+        autospec=True,
+        return_value=False,
+    )
+    def test_manage_pre_38(self, mock_mv):
+        resp = mock.Mock()
+        resp.body = {'volume': copy.deepcopy(VOLUME)}
+        resp.json = mock.Mock(return_value=resp.body)
+        resp.headers = {}
+        resp.status_code = 202
+        self.sess.post = mock.Mock(return_value=resp)
+        sot = volume.Volume.manage(self.sess, host=FAKE_HOST, ref=FAKE_ID)
+        self.assertIsNotNone(sot)
+        url = '/os-volume-manage'
+        body = {
+            'volume': {
+                'host': FAKE_HOST,
+                'ref': FAKE_ID,
+                'name': None,
+                'description': None,
+                'volume_type': None,
+                'availability_zone': None,
+                'metadata': None,
+                'bootable': False,
+            }
+        }
+        self.sess.post.assert_called_with(
+            url, json=body, microversion=sot._max_microversion
+        )
+
+    def test_set_microversion(self):
+        sot = volume.Volume(**VOLUME)
+        self.sess.default_microversion = '3.50'
+        self.assertIsNone(sot.extend(self.sess, '20'))
+
+        url = 'volumes/%s/action' % FAKE_ID
+        body = {"os-extend": {"new_size": "20"}}
+        self.sess.post.assert_called_with(url, json=body, microversion="3.50")
