@@ -11,15 +11,14 @@
 # under the License.
 
 import os
+from unittest import mock
 
 import fixtures
 from keystoneauth1 import session
-import mock
-from testtools import matchers
 
+import openstack.config
 from openstack import connection
 from openstack import proxy
-import openstack.config
 from openstack import service_description
 from openstack.tests import fakes
 from openstack.tests.unit import base
@@ -70,9 +69,13 @@ clouds:
       password: {password}
       project_name: {project}
     cacert: {cacert}
-""".format(auth_url=CONFIG_AUTH_URL, username=CONFIG_USERNAME,
-           password=CONFIG_PASSWORD, project=CONFIG_PROJECT,
-           cacert=CONFIG_CACERT)
+""".format(
+    auth_url=CONFIG_AUTH_URL,
+    username=CONFIG_USERNAME,
+    password=CONFIG_PASSWORD,
+    project=CONFIG_PROJECT,
+    cacert=CONFIG_CACERT,
+)
 
 VENDOR_CONFIG = """
 {{
@@ -96,9 +99,8 @@ public-clouds:
 
 
 class _TestConnectionBase(base.TestCase):
-
     def setUp(self):
-        super(_TestConnectionBase, self).setUp()
+        super().setUp()
         # Create a temporary directory where our test config will live
         # and insert it into the search path via OS_CLIENT_CONFIG_FILE.
         config_dir = self.useFixture(fixtures.TempDir()).path
@@ -107,8 +109,9 @@ class _TestConnectionBase(base.TestCase):
         with open(config_path, "w") as conf:
             conf.write(CLOUD_CONFIG)
 
-        self.useFixture(fixtures.EnvironmentVariable(
-            "OS_CLIENT_CONFIG_FILE", config_path))
+        self.useFixture(
+            fixtures.EnvironmentVariable("OS_CLIENT_CONFIG_FILE", config_path)
+        )
         self.use_keystone_v2()
 
 
@@ -152,104 +155,124 @@ class TestConnection(_TestConnectionBase):
         #                  conn.workflow.__class__.__module__)
 
     def test_create_unknown_proxy(self):
-        self.register_uris([
-            self.get_placement_discovery_mock_dict(),
-        ])
+        self.register_uris(
+            [
+                self.get_placement_discovery_mock_dict(),
+            ]
+        )
 
         def closure():
             return self.cloud.placement
 
-        self.assertThat(
-            closure,
-            matchers.Warnings(matchers.HasLength(0)))
-
-        self.assertIsInstance(
-            self.cloud.placement,
-            proxy.Proxy)
-
+        self.assertIsInstance(self.cloud.placement, proxy.Proxy)
         self.assert_calls()
 
     def test_create_connection_version_param_default(self):
         c1 = connection.Connection(cloud='sample-cloud')
         conn = connection.Connection(session=c1.session)
-        self.assertEqual('openstack.identity.v3._proxy',
-                         conn.identity.__class__.__module__)
+        self.assertEqual(
+            'openstack.identity.v3._proxy', conn.identity.__class__.__module__
+        )
 
     def test_create_connection_version_param_string(self):
         c1 = connection.Connection(cloud='sample-cloud')
         conn = connection.Connection(
-            session=c1.session, identity_api_version='2')
-        self.assertEqual('openstack.identity.v2._proxy',
-                         conn.identity.__class__.__module__)
+            session=c1.session, identity_api_version='2'
+        )
+        self.assertEqual(
+            'openstack.identity.v2._proxy', conn.identity.__class__.__module__
+        )
 
     def test_create_connection_version_param_int(self):
         c1 = connection.Connection(cloud='sample-cloud')
         conn = connection.Connection(
-            session=c1.session, identity_api_version=3)
-        self.assertEqual('openstack.identity.v3._proxy',
-                         conn.identity.__class__.__module__)
+            session=c1.session, identity_api_version=3
+        )
+        self.assertEqual(
+            'openstack.identity.v3._proxy', conn.identity.__class__.__module__
+        )
 
     def test_create_connection_version_param_bogus(self):
         c1 = connection.Connection(cloud='sample-cloud')
         conn = connection.Connection(
-            session=c1.session, identity_api_version='red')
+            session=c1.session, identity_api_version='red'
+        )
         # TODO(mordred) This is obviously silly behavior
-        self.assertEqual('openstack.identity.v3._proxy',
-                         conn.identity.__class__.__module__)
+        self.assertEqual(
+            'openstack.identity.v3._proxy', conn.identity.__class__.__module__
+        )
 
     def test_from_config_given_config(self):
-        cloud_region = (openstack.config.OpenStackConfig().
-                        get_one("sample-cloud"))
+        cloud_region = openstack.config.OpenStackConfig().get_one(
+            "sample-cloud"
+        )
 
         sot = connection.from_config(config=cloud_region)
 
-        self.assertEqual(CONFIG_USERNAME,
-                         sot.config.config['auth']['username'])
-        self.assertEqual(CONFIG_PASSWORD,
-                         sot.config.config['auth']['password'])
-        self.assertEqual(CONFIG_AUTH_URL,
-                         sot.config.config['auth']['auth_url'])
-        self.assertEqual(CONFIG_PROJECT,
-                         sot.config.config['auth']['project_name'])
+        self.assertEqual(
+            CONFIG_USERNAME, sot.config.config['auth']['username']
+        )
+        self.assertEqual(
+            CONFIG_PASSWORD, sot.config.config['auth']['password']
+        )
+        self.assertEqual(
+            CONFIG_AUTH_URL, sot.config.config['auth']['auth_url']
+        )
+        self.assertEqual(
+            CONFIG_PROJECT, sot.config.config['auth']['project_name']
+        )
 
     def test_from_config_given_cloud(self):
         sot = connection.from_config(cloud="sample-cloud")
 
-        self.assertEqual(CONFIG_USERNAME,
-                         sot.config.config['auth']['username'])
-        self.assertEqual(CONFIG_PASSWORD,
-                         sot.config.config['auth']['password'])
-        self.assertEqual(CONFIG_AUTH_URL,
-                         sot.config.config['auth']['auth_url'])
-        self.assertEqual(CONFIG_PROJECT,
-                         sot.config.config['auth']['project_name'])
+        self.assertEqual(
+            CONFIG_USERNAME, sot.config.config['auth']['username']
+        )
+        self.assertEqual(
+            CONFIG_PASSWORD, sot.config.config['auth']['password']
+        )
+        self.assertEqual(
+            CONFIG_AUTH_URL, sot.config.config['auth']['auth_url']
+        )
+        self.assertEqual(
+            CONFIG_PROJECT, sot.config.config['auth']['project_name']
+        )
 
     def test_from_config_given_cloud_config(self):
-        cloud_region = (openstack.config.OpenStackConfig().
-                        get_one("sample-cloud"))
+        cloud_region = openstack.config.OpenStackConfig().get_one(
+            "sample-cloud"
+        )
 
         sot = connection.from_config(cloud_config=cloud_region)
 
-        self.assertEqual(CONFIG_USERNAME,
-                         sot.config.config['auth']['username'])
-        self.assertEqual(CONFIG_PASSWORD,
-                         sot.config.config['auth']['password'])
-        self.assertEqual(CONFIG_AUTH_URL,
-                         sot.config.config['auth']['auth_url'])
-        self.assertEqual(CONFIG_PROJECT,
-                         sot.config.config['auth']['project_name'])
+        self.assertEqual(
+            CONFIG_USERNAME, sot.config.config['auth']['username']
+        )
+        self.assertEqual(
+            CONFIG_PASSWORD, sot.config.config['auth']['password']
+        )
+        self.assertEqual(
+            CONFIG_AUTH_URL, sot.config.config['auth']['auth_url']
+        )
+        self.assertEqual(
+            CONFIG_PROJECT, sot.config.config['auth']['project_name']
+        )
 
     def test_from_config_given_cloud_name(self):
         sot = connection.from_config(cloud_name="sample-cloud")
 
-        self.assertEqual(CONFIG_USERNAME,
-                         sot.config.config['auth']['username'])
-        self.assertEqual(CONFIG_PASSWORD,
-                         sot.config.config['auth']['password'])
-        self.assertEqual(CONFIG_AUTH_URL,
-                         sot.config.config['auth']['auth_url'])
-        self.assertEqual(CONFIG_PROJECT,
-                         sot.config.config['auth']['project_name'])
+        self.assertEqual(
+            CONFIG_USERNAME, sot.config.config['auth']['username']
+        )
+        self.assertEqual(
+            CONFIG_PASSWORD, sot.config.config['auth']['password']
+        )
+        self.assertEqual(
+            CONFIG_AUTH_URL, sot.config.config['auth']['auth_url']
+        )
+        self.assertEqual(
+            CONFIG_PROJECT, sot.config.config['auth']['project_name']
+        )
 
     def test_from_config_verify(self):
         sot = connection.from_config(cloud="insecure-cloud")
@@ -268,29 +291,35 @@ class TestOsloConfig(_TestConnectionBase):
     def test_from_conf(self):
         c1 = connection.Connection(cloud='sample-cloud')
         conn = connection.Connection(
-            session=c1.session, oslo_conf=self._load_ks_cfg_opts())
+            session=c1.session, oslo_conf=self._load_ks_cfg_opts()
+        )
         # There was no config for keystone
         self.assertIsInstance(
-            conn.identity, service_description._ServiceDisabledProxyShim)
+            conn.identity, service_description._ServiceDisabledProxyShim
+        )
         # But nova was in there
-        self.assertEqual('openstack.compute.v2._proxy',
-                         conn.compute.__class__.__module__)
+        self.assertEqual(
+            'openstack.compute.v2._proxy', conn.compute.__class__.__module__
+        )
 
     def test_from_conf_filter_service_types(self):
         c1 = connection.Connection(cloud='sample-cloud')
         conn = connection.Connection(
-            session=c1.session, oslo_conf=self._load_ks_cfg_opts(),
-            service_types={'orchestration', 'i-am-ignored'})
+            session=c1.session,
+            oslo_conf=self._load_ks_cfg_opts(),
+            service_types={'orchestration', 'i-am-ignored'},
+        )
         # There was no config for keystone
         self.assertIsInstance(
-            conn.identity, service_description._ServiceDisabledProxyShim)
+            conn.identity, service_description._ServiceDisabledProxyShim
+        )
         # Nova was in there, but disabled because not requested
         self.assertIsInstance(
-            conn.compute, service_description._ServiceDisabledProxyShim)
+            conn.compute, service_description._ServiceDisabledProxyShim
+        )
 
 
 class TestNetworkConnection(base.TestCase):
-
     # Verify that if the catalog has the suffix we don't mess things up.
     def test_network_proxy(self):
         self.os_fixture.v3_token.remove_service('network')
@@ -298,15 +327,18 @@ class TestNetworkConnection(base.TestCase):
         svc.add_endpoint(
             interface='public',
             url='https://network.example.com/v2.0',
-            region='RegionOne')
+            region='RegionOne',
+        )
         self.use_keystone_v3()
         self.assertEqual(
             'openstack.network.v2._proxy',
-            self.cloud.network.__class__.__module__)
+            self.cloud.network.__class__.__module__,
+        )
         self.assert_calls()
         self.assertEqual(
             "https://network.example.com/v2.0",
-            self.cloud.network.get_endpoint())
+            self.cloud.network.get_endpoint(),
+        )
 
 
 class TestNetworkConnectionSuffix(base.TestCase):
@@ -316,15 +348,16 @@ class TestNetworkConnectionSuffix(base.TestCase):
     def test_network_proxy(self):
         self.assertEqual(
             'openstack.network.v2._proxy',
-            self.cloud.network.__class__.__module__)
+            self.cloud.network.__class__.__module__,
+        )
         self.assert_calls()
         self.assertEqual(
             "https://network.example.com/v2.0",
-            self.cloud.network.get_endpoint())
+            self.cloud.network.get_endpoint(),
+        )
 
 
 class TestAuthorize(base.TestCase):
-
     def test_authorize_works(self):
         res = self.cloud.authorize()
         self.assertEqual('KeystoneToken-1', res)
@@ -332,18 +365,18 @@ class TestAuthorize(base.TestCase):
     def test_authorize_failure(self):
         self.use_broken_keystone()
 
-        self.assertRaises(openstack.exceptions.SDKException,
-                          self.cloud.authorize)
+        self.assertRaises(
+            openstack.exceptions.SDKException, self.cloud.authorize
+        )
 
 
 class TestNewService(base.TestCase):
-
     def test_add_service_v1(self):
         svc = self.os_fixture.v3_token.add_service('fake')
         svc.add_endpoint(
             interface='public',
             region='RegionOne',
-            url='https://fake.example.com/v1/{0}'.format(fakes.PROJECT_ID),
+            url=f'https://fake.example.com/v1/{fakes.PROJECT_ID}',
         )
         self.use_keystone_v3()
         conn = self.cloud
@@ -355,21 +388,30 @@ class TestNewService(base.TestCase):
         # Ensure no discovery calls made
         self.assertEqual(0, len(self.adapter.request_history))
 
-        self.register_uris([
-            dict(method='GET',
-                 uri='https://fake.example.com',
-                 status_code=404),
-            dict(method='GET',
-                 uri='https://fake.example.com/v1/',
-                 status_code=404),
-            dict(method='GET',
-                 uri=self.get_mock_url('fake'),
-                 status_code=404),
-        ])
+        self.register_uris(
+            [
+                dict(
+                    method='GET',
+                    uri='https://fake.example.com',
+                    status_code=404,
+                ),
+                dict(
+                    method='GET',
+                    uri='https://fake.example.com/v1/',
+                    status_code=404,
+                ),
+                dict(
+                    method='GET',
+                    uri=self.get_mock_url('fake'),
+                    status_code=404,
+                ),
+            ]
+        )
 
         self.assertEqual(
             'openstack.tests.unit.fake.v1._proxy',
-            conn.fake.__class__.__module__)
+            conn.fake.__class__.__module__,
+        )
         self.assertTrue(conn.fake.dummy())
 
     def test_add_service_v2(self):
@@ -377,22 +419,30 @@ class TestNewService(base.TestCase):
         svc.add_endpoint(
             interface='public',
             region='RegionOne',
-            url='https://fake.example.com/v2/{0}'.format(fakes.PROJECT_ID),
+            url=f'https://fake.example.com/v2/{fakes.PROJECT_ID}',
         )
         self.use_keystone_v3()
         conn = self.cloud
 
-        self.register_uris([
-            dict(method='GET',
-                 uri='https://fake.example.com',
-                 status_code=404),
-            dict(method='GET',
-                 uri='https://fake.example.com/v2/',
-                 status_code=404),
-            dict(method='GET',
-                 uri=self.get_mock_url('fake'),
-                 status_code=404),
-        ])
+        self.register_uris(
+            [
+                dict(
+                    method='GET',
+                    uri='https://fake.example.com',
+                    status_code=404,
+                ),
+                dict(
+                    method='GET',
+                    uri='https://fake.example.com/v2/',
+                    status_code=404,
+                ),
+                dict(
+                    method='GET',
+                    uri=self.get_mock_url('fake'),
+                    status_code=404,
+                ),
+            ]
+        )
 
         service = fake_service.FakeService('fake')
 
@@ -400,7 +450,8 @@ class TestNewService(base.TestCase):
 
         self.assertEqual(
             'openstack.tests.unit.fake.v2._proxy',
-            conn.fake.__class__.__module__)
+            conn.fake.__class__.__module__,
+        )
         self.assertFalse(conn.fake.dummy())
 
     def test_replace_system_service(self):
@@ -408,7 +459,7 @@ class TestNewService(base.TestCase):
         svc.add_endpoint(
             interface='public',
             region='RegionOne',
-            url='https://fake.example.com/v2/{0}'.format(fakes.PROJECT_ID),
+            url=f'https://fake.example.com/v2/{fakes.PROJECT_ID}',
         )
         self.use_keystone_v3()
         conn = self.cloud
@@ -416,17 +467,25 @@ class TestNewService(base.TestCase):
         # delete native dns service
         delattr(conn, 'dns')
 
-        self.register_uris([
-            dict(method='GET',
-                 uri='https://fake.example.com',
-                 status_code=404),
-            dict(method='GET',
-                 uri='https://fake.example.com/v2/',
-                 status_code=404),
-            dict(method='GET',
-                 uri=self.get_mock_url('fake'),
-                 status_code=404),
-        ])
+        self.register_uris(
+            [
+                dict(
+                    method='GET',
+                    uri='https://fake.example.com',
+                    status_code=404,
+                ),
+                dict(
+                    method='GET',
+                    uri='https://fake.example.com/v2/',
+                    status_code=404,
+                ),
+                dict(
+                    method='GET',
+                    uri=self.get_mock_url('fake'),
+                    status_code=404,
+                ),
+            ]
+        )
 
         # add fake service with alias 'DNS'
         service = fake_service.FakeService('fake', aliases=['dns'])
@@ -441,9 +500,8 @@ def vendor_hook(conn):
 
 
 class TestVendorProfile(base.TestCase):
-
     def setUp(self):
-        super(TestVendorProfile, self).setUp()
+        super().setUp()
         # Create a temporary directory where our test config will live
         # and insert it into the search path via OS_CLIENT_CONFIG_FILE.
         config_dir = self.useFixture(fixtures.TempDir()).path
@@ -456,15 +514,16 @@ class TestVendorProfile(base.TestCase):
         with open(public_clouds, "w") as conf:
             conf.write(PUBLIC_CLOUDS_YAML)
 
-        self.useFixture(fixtures.EnvironmentVariable(
-            "OS_CLIENT_CONFIG_FILE", config_path))
+        self.useFixture(
+            fixtures.EnvironmentVariable("OS_CLIENT_CONFIG_FILE", config_path)
+        )
         self.use_keystone_v2()
 
         self.config = openstack.config.loader.OpenStackConfig(
-            vendor_files=[public_clouds])
+            vendor_files=[public_clouds]
+        )
 
     def test_conn_from_profile(self):
-
         self.cloud = self.config.get_one(cloud='profiled-cloud')
 
         conn = connection.Connection(config=self.cloud)
@@ -472,7 +531,6 @@ class TestVendorProfile(base.TestCase):
         self.assertIsNotNone(conn)
 
     def test_hook_from_profile(self):
-
         self.cloud = self.config.get_one(cloud='profiled-cloud')
 
         conn = connection.Connection(config=self.cloud)
@@ -480,19 +538,17 @@ class TestVendorProfile(base.TestCase):
         self.assertEqual('test_val', conn.test)
 
     def test_hook_from_connection_param(self):
-
         conn = connection.Connection(
             cloud='sample-cloud',
-            vendor_hook='openstack.tests.unit.test_connection:vendor_hook'
+            vendor_hook='openstack.tests.unit.test_connection:vendor_hook',
         )
 
         self.assertEqual('test_val', conn.test)
 
     def test_hook_from_connection_ignore_missing(self):
-
         conn = connection.Connection(
             cloud='sample-cloud',
-            vendor_hook='openstack.tests.unit.test_connection:missing'
+            vendor_hook='openstack.tests.unit.test_connection:missing',
         )
 
         self.assertIsNotNone(conn)
